const { Sequelize, DataTypes, INTEGER } = require('sequelize');
const sequelize = require('./sequelizeConnection');

const Edition = sequelize.define('sc_edition', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  locale: {
    type: DataTypes.TEXT,
    allowNull: false,
  },
  created_date: {
    type: DataTypes.DATE,
    allowNull: false,
    defaultValue: Sequelize.NOW,
  },
  updated_date: {
    type: DataTypes.DATE,
    allowNull: false,
    defaultValue: Sequelize.NOW,
  },
  created_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  updated_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  title: {
    type: DataTypes.TEXT,
  },
  publish_date: {
    type: DataTypes.DATE
  },
  featured_challenge_id: {
    type: DataTypes.INTEGER,
  }
}, {
  freezeTableName: true,
  timestamps: false,
});

const Challenge = sequelize.define('sc_challenge', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  locale: {
    type: DataTypes.TEXT,
    allowNull: false,
  },
  thumbnail: {
    type: DataTypes.JSONB,
  },
  steps: {
    type: DataTypes.JSONB,
  },
  materials: {
    type: DataTypes.JSONB,
  },
  created_date: {
    type: DataTypes.DATE,
    allowNull: false,
    defaultValue: Sequelize.NOW,
  },
  updated_date: {
    type: DataTypes.DATE,
    allowNull: false,
    defaultValue: Sequelize.NOW,
  },
  created_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  updated_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  title: {
    type: DataTypes.TEXT,
  },
  video_id: {
    type: DataTypes.INTEGER,
  },
  description: {
    type: DataTypes.TEXT,
  },
  publish_date: {
    type: DataTypes.DATE
  },
  expire_date: {
    type: DataTypes.DATE
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const Question = sequelize.define('sc_question', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  locale: {
    type: DataTypes.TEXT,
    allowNull: false,
  },
  created_date: {
    type: DataTypes.DATE,
    allowNull: false,
  },
  updated_date: {
    type: DataTypes.DATE,
    allowNull: false,
  },
  created_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  updated_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  prompt: {
    type: DataTypes.TEXT,
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const Choice = sequelize.define('sc_choice', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  locale: {
    type: DataTypes.TEXT,
    allowNull: false,
  },
  created_date: {
    type: DataTypes.DATE,
    allowNull: false,
  },
  updated_date: {
    type: DataTypes.DATE,
    allowNull: false,
  },
  created_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  updated_by_editorial_user: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
  label: {
    type: DataTypes.TEXT,
  },
  message: {
    type: DataTypes.TEXT,
  },
}, {
  freezeTableName: true,
  timestamps: false,
}, {
  hooks: {
    beforeValidate: (choice, options) => {
      choice.created_date = sequelize.fn('NOW');
      choice.updated_date = sequelize.fn('NOW');
    },
  }
});

const EditionQuestion = sequelize.define('sc_edition_question', {
  sc_edition_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_edition',
      key: 'id'
    }
  },
  sc_question_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_question',
      key: 'id'
    }
  },
  sort_num: {
    type: DataTypes.INTEGER,
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const QuestionChoice = sequelize.define('sc_question_choice', {
  sc_choice_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_choice',
      key: 'id'
    }
  },
  sc_question_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_question',
      key: 'id'
    }
  },
  sort_num: {
    type: DataTypes.INTEGER,
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const User = sequelize.define('sc_user', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  snap_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const AssetChoice = sequelize.define('sc_asset_choice', {
  id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    autoIncrement: true,
    primaryKey: true,
  },
  sticker: {
    type: DataTypes.JSONB,
  },
  sc_choice_id_hash: {
    type: DataTypes.JSONB,
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const SnapUserChoice = sequelize.define('sc_snap_user_choice', {
  sc_choice_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_choice',
      key: 'id'
    }
  },
  sc_question_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_question',
      key: 'id'
    }
  },
  sc_user_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_user',
      key: 'id'
    }
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

const SnapUserChallenge = sequelize.define('sc_snap_user_challenge', {
  sc_challenge_id: {
    type: DataTypes.INTEGER,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_challenge',
      key: 'id'
    }
  },
  sc_user_id: {
    type: DataTypes.STRING,
    allowNull: false,
    primaryKey: true,
    references: {
      model: 'sc_user',
      key: 'id'
    }
  },
}, {
  freezeTableName: true,
  timestamps: false,
});

User.Choices = User.belongsToMany(Choice, {
  through: SnapUserChoice,
  foreignKey: 'sc_user_id',
  as: 'choices'
});

Choice.Users = Choice.belongsToMany(User, {
  through: SnapUserChoice,
  foreignKey: 'sc_choice_id',
  as: 'users'
});

User.Questions = User.belongsToMany(Question, {
  through: SnapUserChoice,
  foreignKey: 'sc_user_id',
  as: 'questions'
});

Question.Users = Question.belongsToMany(User, {
  through: SnapUserChoice,
  foreignKey: 'sc_question_id',
  as: 'users'
});

User.Challenges = User.belongsToMany(Challenge, {
  through: SnapUserChallenge,
  foreignKey: 'sc_user_id',
  as: 'challenges'
});

Challenge.Users = Challenge.belongsToMany(User, {
  through: SnapUserChallenge,
  foreignKey: 'sc_challenge_id',
  as: 'users'
});

Edition.FeaturedChallenge = Edition.belongsTo(Challenge, {
  foreignKey: 'featured_challenge_id',
  as: 'featured_challenge'
});

Challenge.Editions = Challenge.hasMany(Edition, {
  foreignKey: 'featured_challenge_id',
  as: 'featured_in_editions'
});

Edition.Questions = Edition.belongsToMany(Question, {
  through: EditionQuestion,
  foreignKey: 'sc_edition_id',
  as: 'questions'
});

Choice.Questions = Choice.belongsToMany(Question, {
  through: QuestionChoice,
  foreignKey: 'sc_choice_id',
  as: 'questions'
});

Question.Choices = Question.belongsToMany(Choice, {
  through: QuestionChoice,
  foreignKey: 'sc_question_id',
  as: 'choices'
});


Question.Editions = Question.belongsToMany(Edition, {
  through: EditionQuestion,
  foreignKey: 'sc_question_id',
  as: 'editions'
});

module.exports = {
  Challenge,
  AssetChoice,
  SnapUserChallenge,
  SnapUserChoice,
  QuestionChoice,
  Edition,
  User,
  Question,
  Choice,
  sequelize,
};